@@ -1,4 +1,4 @@
-//  $Id: schema_rpsl.cc 1.5 Tue, 28 Sep 1999 18:28:57 -0700 cengiz $
+//  $Id: schema_rpsl.cc 1.7 Wed, 17 Nov 1999 12:16:46 -0800 fabio $
 //
 //  Copyright (c) 1994 by the University of Southern California
 //  All rights reserved.
@@ -365,9 +365,8 @@ void Schema::initializeBase() {
    initializeBaseDone = true;
 
    AttrClass *classClass = new AttrClass(strdup("class"));
-   AttrClass *clss = NULL;
-
    addClass(classClass);
+
    classClass->addAttr(new AttrAttr(strdup("class"), ATTR_GENERIC, 
 				    RPType::newRPType("rpsl_word"),
 				    AttrAttr::LOOKUP));
@@ -405,12 +404,13 @@ List<Object> *Schema::initializeFromFile(char *filename) {
 	 delete o;
       else {
 	 if (! strcasecmp(o->type->name, "class")) {
-	    AttrGenericIterator<ItemWORD> cname(o, "class");
-	    AttrIterator<AttrAttr>  attrItr(o, "attr");
+	    Object *o2 = new Object(*o);
+	    AttrGenericIterator<ItemWORD> cname(o2, "class");
+	    AttrIterator<AttrAttr>  attrItr(o2, "attr");
 	    char *name = cname.first()->word;
 
 	    addClass(clss = new AttrClass(strdup(name)));
-	    clss->object = o;
+	    clss->object = o2;
 	    //cerr << "Reading "<< name << " class definition..." << endl;
 	    
 	    for (attr = attrItr.first(); attr; attr = attrItr.next())
@@ -432,21 +432,24 @@ Object *Schema::initializeFromString(char *class_text, char *lookFor) {
    Object *result = (Object *) NULL;
    AttrClass *clss = NULL;
    int count = 0;
+   bool keepO = false;
 
    initializeBase();
 
    do {
       count++;
+      keepO = false;
       o = new Object;
-      if (count == 2 && ! dictionary)
+      if (count == 2 && ! dictionary) {
+	 keepO = true;
 	 dictionary = o;
+      }
 
       o->scan(class_text, strlen(class_text));
 
-      if (o->has_error)
-	 delete o;
-      else {
+      if (! o->has_error) {
 	 if (!strcasecmp(o->type->name, "class")) {
+	    keepO = true;
 	    AttrGenericIterator<ItemWORD> cname(o, "class");
 	    AttrIterator<AttrAttr>  attrItr(o, "attr");
 	    char *name = cname.first()->word;
@@ -457,14 +460,17 @@ Object *Schema::initializeFromString(char *class_text, char *lookFor) {
 
 	    for (attr = attrItr.first(); attr; attr = attrItr.next())
 	       clss->addAttr((AttrAttr *) attr);
-
-	    //delete o;
 	 }
 
-	 if (lookFor && ! strcasecmp(o->type->name, lookFor))
+	 if (lookFor && ! strcasecmp(o->type->name, lookFor)) {
+	    keepO = true;
 	    result = o;
+	 }
       }
 
+      if (!keepO)
+	 delete o;
+
       class_text = strstr(class_text, "\n\n");
       if (class_text)
 	 class_text += 2;
